
//	base code for simple applications
//  Copyright (c) 2002 Takeo Igarashi 

import java.awt.event.*;
import java.awt.*;
import java.util.*;


class MainPanel extends Panel implements ButtonsInterface
{

	// location of the screen within the virtual field.
	public double center_x = 0;
	public double center_y = 0;
	// zoom level.  zoom-in = scale-small, zoom-out = scale-large
	public double scale = 1;
	
	// Box objects on the field.
	public Vector boxes = new Vector();
							  

	
	//
	// initialize
	//
	MainPanel(){
		MouseDispatcher mouseDispatcher = new MouseDispatcher();
		addMouseListener(mouseDispatcher);
		addMouseMotionListener(mouseDispatcher);
		
		init();
	}
	
	
	
	//
	// set buttons
	//
	public void initializeButtons(Buttons buttons){
		buttons.addButton("quit");
		buttons.addButton("init");
	}
	public void buttonPressed(String label){
		if (label.equals("quit"))
			System.exit(0);
		else if (label.equals("init"))
			init();
	}
	
	
	
	
	// initialize
	
	public void init(){
		scale = 1;
		center_x = 0;
		center_y = 0;
		
		boxes = new Vector();
		boxes.addElement(new Box(100,140, 120,80, Color.blue));
		boxes.addElement(new Box(180,260, 200,120, Color.red));
		boxes.addElement(new Box(400,240, 120,60, Color.green));
		
		repaint();
	}
	
	
	
	

	

	
	//
	// paint screen
	//
	Image buffered_image = null;
	public void update(Graphics g) {
		paint(g);
	}
	public void paint(Graphics g) {

		//double buffering
		if (buffered_image == null)
			buffered_image = createImage(getSize().width, getSize().height);
		Graphics bg = buffered_image.getGraphics();

		// clear
		bg.setColor(Color.white);
		bg.fillRect(0,0, getSize().width, getSize().height);

		// paint code!
		paint_boxes(bg);
		paint_zoom_center(bg);
		paint_usage(bg);
						  
		//double buffering
		bg.dispose();
      
		g.drawImage(buffered_image,0,0,this);

	}
	
	public void paint_boxes(Graphics g){
		for(int i=0; i<boxes.size(); i++){
			Box box = (Box) boxes.elementAt(i);
			box.paint(g, center_x, center_y, scale);
		}
	}
	
	public void paint_usage(Graphics g){
		g.setColor(Color.black);
		g.drawString("Scroll: left button, drag background", 10, 20);
		g.drawString("Zoom: right button, drag horizontally", 10, 40);
		g.drawString("Drag Box: left button, drag box", 10, 60);
		g.drawString("Wiggle Box: left button, click box", 10, 80);
	}
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	// drag box
	
	Point prev_p;
	Box dragged_box = null;
	public  boolean start_dragging(Point p){
		dragged_box = pick_box(p);
		prev_p = p;
		
		return (dragged_box != null);
	}
	public void dragging(Point p){
		dragged_box.move_relative( (p.x-prev_p.x)/scale, (p.y-prev_p.y)/scale);
		prev_p = p;
		repaint();
	}
	
	public Box pick_box(Point p){
		for(int i=0; i<boxes.size(); i++){
			Box box = (Box) boxes.elementAt(i);
			if (box.contains(p))
				return box;
		}
		return null;
	}
	

	
	
	
	
	// zoom
	
	Point start_p;
	public void start_zooming(Point p){
		prev_p = p;
		start_p = p;
	}
	public void zooming(Point p){
		double prev_scale = scale;
		scale *= Math.pow(1.02, (p.x - prev_p.x));
		
		center_x += start_p.x * (scale / prev_scale - 1) / scale;
		center_y += start_p.y * (scale / prev_scale - 1) / scale;
		
		prev_p = p;
		repaint();
	}
	public void paint_zoom_center(Graphics g){
		if (operation_status == ZOOMING){
			g.setColor(Color.pink);
			g.drawOval(start_p.x - 10, start_p.y - 10, 20, 20);
			g.drawLine(start_p.x, start_p.y - 20, start_p.x, start_p.y + 20);
			g.drawLine(start_p.x -20, start_p.y, start_p.x+20, start_p.y);
		}
	}
	
	
	
	
	
	// scroll
	
	public void start_scrolling(Point p){
		prev_p = p;
	}
	public void scrolling(Point p){
		center_x += (prev_p.x-p.x)/scale;
		center_y += (prev_p.y-p.y)/scale;
		prev_p = p;
		repaint();
	}

	
	
	
	// animation
	
	public void clicked(Point p){
		Box box = pick_box(p);
		if (box != null)
			box.start_animation(this);
	}
	
	
	
	
	
	
	
   
	//   
	// central event dispatcher
	//
	public int operation_status;
	public static final int NONE = 0;
	public static final int ZOOMING = 1;
	public static final int SCROLLING = 2;
	public static final int DRAGGING = 3;
	
	public class MouseDispatcher 
		extends MouseAdapter implements MouseMotionListener{

  		public void mouseMoved(MouseEvent e) {
  		}
  		public void mousePressed(MouseEvent e) {
			Point p = e.getPoint();
			if ((e.getModifiers() & e.BUTTON3_MASK) != 0){
				operation_status = ZOOMING;
				start_zooming(p);
			}
			else {
				if (start_dragging(p)){
					operation_status = DRAGGING;
				}
				else {
					operation_status = SCROLLING;
					start_scrolling(p);
				}
			}
		}
  		public void mouseDragged(MouseEvent e) {
			Point p = e.getPoint();
			switch(operation_status){
			case ZOOMING:
				zooming(p);
				break;
			case SCROLLING:
				scrolling(p);
				break;
			case DRAGGING:
				dragging(p);
				break;
			}
		}	
		public void mouseReleased(MouseEvent e) {
			operation_status = NONE;
			repaint();
		}
		
		public void mouseClicked(MouseEvent e){
			clicked(e.getPoint());	
		}
    }
}